#!/usr/local/bin/python3.13
###############################################################################
#
#  Project:  GDAL samples
#  Purpose:  Create a virtual directory
#  Author:   Even Rouault <even.rouault at spatialys.com>
#
###############################################################################
#  Copyright (c) 2017, Even Rouault <even.rouault at spatialys.com>
#
# SPDX-License-Identifier: MIT
###############################################################################

import sys

from osgeo import gdal


def Usage():
    print("Usage: gdal_mkdir filename")
    return 2


def gdal_mkdir(argv, progress=None):
    # pylint: disable=unused-argument
    filename = None

    argv = gdal.GeneralCmdLineProcessor(argv)
    if argv is None:
        return -1

    for i in range(1, len(argv)):
        if filename is None:
            filename = argv[i]
        elif argv[i][0] == "-":
            print("Unexpected option : %s" % argv[i])
            return Usage()
        else:
            print("Unexpected option : %s" % argv[i])
            return Usage()

    if filename is None:
        return Usage()

    ret = gdal.Mkdir(filename, int("0755", 8))
    if ret != 0:
        print("Creation failed")
    return ret


def main(argv=sys.argv):
    return gdal_mkdir(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL
#  Purpose:  Dump JPEG2000 file structure
#  Author:   Even Rouault, <even dot rouault at spatialys dot com>
#
# ******************************************************************************
#  Copyright (c) 2015, European Union (European Environment Agency)
#  Copyright (c) 2015, European Union Satellite Centre
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import sys

from osgeo import gdal


def Usage():
    print("Usage:  dump_jp2 [-dump_gmljp2 out.xml|-] [-dump_crsdictionary out.xml|-]")
    print("                 [-extract_all_xml_boxes filename_prefix]")
    print("                 test.jp2")
    print("")
    print("Options (all are exclusive of the regular dump):")
    print("")
    print("-dump_gmljp2: Writes the content of the GMLJP2 box in the specified")
    print('              file, or on the console if "-" syntax is used.')
    print(
        "-dump_crsdictionary: Writes the content of the GML CRS dictionary box in the specified"
    )
    print('                     file, or on the console if "-" syntax is used.')
    print(
        "-extract_all_xml_boxes: Extract all XML boxes in separate files, and prefix each filename"
    )
    print(
        "                        with the supplied prefix. gmljp2://xml/ link will be replaced by"
    )
    print("                        links to on-disk files.")

    return 2


def dump_gmljp2(filename, out_gmljp2):
    ds = gdal.Open(filename)
    if ds is None:
        print("Cannot open %s" % filename)
        return 1
    mdd = ds.GetMetadata("xml:gml.root-instance")
    if mdd is None:
        print("No GMLJP2 content found in %s" % filename)
        return 1
    if out_gmljp2 == "-":
        print(mdd[0])
    else:
        f = open(out_gmljp2, "wt")
        f.write(mdd[0])
        f.close()
        print("INFO: %s written with content of GMLJP2 box" % out_gmljp2)
    return 0


def dump_crsdictionary(filename, out_crsdictionary):
    ds = gdal.Open(filename)
    if ds is None:
        print("Cannot open %s" % filename)
        return 1
    mdd_list = ds.GetMetadataDomainList()
    if mdd_list is None:
        mdd_list = []
    for domain in mdd_list:
        if domain.startswith("xml:"):
            mdd_item = ds.GetMetadata(domain)[0]
            if "<Dictionary" in mdd_item or "<gml:Dictionary" in mdd_item:
                if out_crsdictionary == "-":
                    print(mdd_item)
                else:
                    f = open(out_crsdictionary, "wt")
                    f.write(mdd_item)
                    f.close()
                    print(
                        "INFO: %s written with content of CRS dictionary box (%s)"
                        % (out_crsdictionary, domain[4:])
                    )
                return 0

    print("No CRS dictionary content found in %s" % filename)
    return 1


def extract_all_xml_boxes(filename, prefix):
    ds = gdal.Open(filename)
    if ds is None:
        print("Cannot open %s" % filename)
        return 1
    mdd_list = ds.GetMetadataDomainList()
    if mdd_list is None:
        mdd_list = []
    for domain in mdd_list:
        if domain.startswith("xml:"):
            mdd_item = ds.GetMetadata(domain)[0]
            boxname = domain[4:]
            if boxname == "gml.root-instance":
                boxname = "gml_root_instance.gml"
            out_filename = prefix + boxname

            # Correct references to gmljp2://xml/foo to prefix_foo
            out_content = ""
            pos = 0
            while True:
                new_pos = mdd_item.find("gmljp2://xml/", pos)
                if new_pos < 0:
                    out_content += mdd_item[pos:]
                    break

                # Check that the referenced box really exists
                end_gmljp2_link_space = mdd_item.find(" ", new_pos)
                end_gmljp2_link_double_quote = mdd_item.find('"', new_pos)
                end_gmljp2_link = -1
                if end_gmljp2_link_space >= 0 and end_gmljp2_link_double_quote >= 0:
                    if end_gmljp2_link_space < end_gmljp2_link_double_quote:
                        end_gmljp2_link = end_gmljp2_link_space
                    else:
                        end_gmljp2_link = end_gmljp2_link_double_quote
                elif end_gmljp2_link_space >= 0:
                    end_gmljp2_link = end_gmljp2_link_space
                elif end_gmljp2_link_double_quote >= 0:
                    end_gmljp2_link = end_gmljp2_link_double_quote
                if end_gmljp2_link >= 0:
                    referenced_box = mdd_item[
                        new_pos + len("gmljp2://xml/") : end_gmljp2_link
                    ]
                    if ("xml:" + referenced_box) not in mdd_list:
                        print(
                            "Warning: box %s reference box %s, but the latter is not found"
                            % (boxname, referenced_box)
                        )

                out_content += mdd_item[pos:new_pos]
                out_content += prefix
                pos = new_pos + len("gmljp2://xml/")

            f = open(out_filename, "wt")
            f.write(out_content)
            f.close()
            print("INFO: %s written" % out_filename)

    if not mdd_list:
        print("No XML box found")
        return 1
    return 0


def main(argv=sys.argv):
    i = 1
    out_gmljp2 = None
    out_crsdictionary = None
    extract_all_xml_boxes_prefix = None
    filename = None
    while i < len(argv):
        if argv[i] == "-dump_gmljp2":
            if i >= len(argv) - 1:
                return Usage()
            out_gmljp2 = argv[i + 1]
            i = i + 1
        elif argv[i] == "-dump_crsdictionary":
            if i >= len(argv) - 1:
                return Usage()
            out_crsdictionary = argv[i + 1]
            i = i + 1
        elif argv[i] == "-extract_all_xml_boxes":
            if i >= len(argv) - 1:
                return Usage()
            extract_all_xml_boxes_prefix = argv[i + 1]
            i = i + 1
        elif argv[i][0] == "-":
            return Usage()
        elif filename is None:
            filename = argv[i]
        else:
            return Usage()

        i = i + 1

    if filename is None:
        return Usage()

    if out_gmljp2 or out_crsdictionary or extract_all_xml_boxes_prefix:
        if out_gmljp2:
            if dump_gmljp2(filename, out_gmljp2) != 0:
                return 1
        if out_crsdictionary:
            if dump_crsdictionary(filename, out_crsdictionary) != 0:
                return 1
        if extract_all_xml_boxes_prefix:
            if extract_all_xml_boxes(filename, extract_all_xml_boxes_prefix) != 0:
                return 1
    else:
        s = gdal.GetJPEG2000StructureAsString(filename, ["ALL=YES"])
        print(s)

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# -*- coding: utf-8 -*-
###############################################################################
#
# Project:  GDAL/OGR samples
# Purpose:  Dispatch features into layers according to the value of some fields
#           or the geometry type.
# Author:   Even Rouault <even dot rouault at spatialys.com>
#
###############################################################################
# Copyright (c) 2013, Even Rouault <even dot rouault at spatialys.com>
#
# SPDX-License-Identifier: MIT
###############################################################################

import sys

from osgeo import ogr, osr


def Usage():
    print("Usage: ogr_dispatch.py [-f format] -src name -dst name [-field field]+")
    print("                [-25D_as_2D] [-multi_as_single]")
    print("                [-remove_dispatch_fields] [-prefix_with_layer_name]")
    print("                [-dsco KEY=VALUE]... [-lco KEY=VALUE]... [-a_srs srs_def]")
    print(
        "                [-style_as_field] [-where restricted_where] [-gt n] [-quiet]"
    )
    print("")
    print("Dispatch features into layers according to the value of some fields or the")
    print("geometry type.")
    print("")
    print("Arguments:")
    print(" -f format: name of the driver to use to create the destination dataset")
    print("            (default 'ESRI Shapefile').")
    print(" -src name: name of the source dataset.")
    print(" -dst name: name of the destination dataset (existing or to be created).")
    print(" -field field: name of a field to use to dispatch features. May also")
    print("                        be 'OGR_GEOMETRY'. At least, one occurrence needed.")
    print(" -25D_as_2D: for dispatching, consider 2.5D geometries as 2D.")
    print(" -multi_as_single: for dispatching, consider MULTIPOLYGON as POLYGON and")
    print("                   MULTILINESTRING as LINESTRING.")
    print(
        " -remove_dispatch_fields: remove the dispatch fields from the target layer definitions."
    )
    print(
        " -prefix_with_layer_name: prefix the target layer name with the source layer name."
    )
    print(" -dsco KEY=VALUE: dataset creation option. May be repeated.")
    print(" -lco KEY=VALUE: layer creation option. May be repeated.")
    print(
        " -a_srs srs_def: assign a SRS to the target layers. Source layer SRS is otherwise used."
    )
    print(
        " -style_as_field: add a OGR_STYLE field with the content of the feature style string."
    )
    print(" -where restricted_where: where clause to filter source features.")
    print(" -gt n: group n features per transaction (default 200).")
    print("")
    print("Example :")
    print("  ogr_dispatch.py -src in.dxf -dst out -field Layer -field OGR_GEOMETRY")
    print("")

    return 2


###############################################################################


def EQUAL(a, b):
    return a.lower() == b.lower()


###############################################################
# wkbFlatten()


def wkbFlatten(x):
    return x & (~ogr.wkb25DBit)


###############################################################


class Options:
    def __init__(self):
        self.lco = []
        self.dispatch_fields = []
        self.b25DAs2D = False
        self.bMultiAsSingle = False
        self.bRemoveDispatchFields = False
        self.poOutputSRS = None
        self.bNullifyOutputSRS = False
        self.bPrefixWithLayerName = False
        self.bStyleAsField = False
        self.nGroupTransactions = 200
        self.bQuiet = False


###############################################################
# GeometryTypeToName()


def GeometryTypeToName(eGeomType, options):

    if options.b25DAs2D:
        eGeomType = wkbFlatten(eGeomType)

    if eGeomType == ogr.wkbPoint:
        return "POINT"
    if eGeomType == ogr.wkbLineString:
        return "LINESTRING"
    if eGeomType == ogr.wkbPolygon:
        return "POLYGON"
    if eGeomType == ogr.wkbMultiPoint:
        return "MULTIPOINT"
    if eGeomType == ogr.wkbMultiLineString:
        return "LINESTRING" if options.bMultiAsSingle else "MULTILINESTRING"
    if eGeomType == ogr.wkbMultiPolygon:
        return "POLYGON" if options.bMultiAsSingle else "MULTIPOLYGON"
    if eGeomType == ogr.wkbGeometryCollection:
        return "GEOMETRYCOLLECTION"
    if eGeomType == ogr.wkbPoint25D:
        return "POINT25D"
    if eGeomType == ogr.wkbLineString25D:
        return "LINESTRING25D"
    if eGeomType == ogr.wkbPolygon25D:
        return "POLYGON25D"
    if eGeomType == ogr.wkbMultiPoint25D:
        return "MULTIPOINT25D"
    if eGeomType == ogr.wkbMultiLineString25D:
        return "LINESTRING25D" if options.bMultiAsSingle else "MULTILINESTRING25D"
    if eGeomType == ogr.wkbMultiPolygon25D:
        return "POLYGON25D" if options.bMultiAsSingle else "MULTIPOLYGON25D"
    if eGeomType == ogr.wkbGeometryCollection25D:
        return "GEOMETRYCOLLECTION25D"
    # Shouldn't happen
    return "UNKNOWN"


###############################################################
# get_out_lyr_name()


def get_out_lyr_name(src_lyr, feat, options):
    if options.bPrefixWithLayerName:
        out_lyr_name = src_lyr.GetName()
    else:
        out_lyr_name = ""

    for dispatch_field in options.dispatch_fields:
        if EQUAL(dispatch_field, "OGR_GEOMETRY"):
            geom = feat.GetGeometryRef()
            if geom is None:
                val = "NONE"
            else:
                val = GeometryTypeToName(geom.GetGeometryType(), options)
        else:
            if feat.IsFieldSet(dispatch_field):
                val = feat.GetFieldAsString(dispatch_field)
            else:
                val = "null"

        if out_lyr_name == "":
            out_lyr_name = val
        else:
            out_lyr_name = out_lyr_name + "_" + val

    return out_lyr_name


###############################################################
# get_layer_and_map()


def get_layer_and_map(out_lyr_name, src_lyr, dst_ds, layerMap, geom_type, options):

    if out_lyr_name not in layerMap:
        if options.poOutputSRS is not None or options.bNullifyOutputSRS:
            srs = options.poOutputSRS
        else:
            srs = src_lyr.GetSpatialRef()
        out_lyr = dst_ds.GetLayerByName(out_lyr_name)
        if out_lyr is None:
            if not options.bQuiet:
                print("Creating layer %s" % out_lyr_name)
            out_lyr = dst_ds.CreateLayer(
                out_lyr_name, srs=srs, geom_type=geom_type, options=options.lco
            )
            if out_lyr is None:
                return 1
            src_field_count = src_lyr.GetLayerDefn().GetFieldCount()
            panMap = [-1 for i in range(src_field_count)]
            for i in range(src_field_count):
                field_defn = src_lyr.GetLayerDefn().GetFieldDefn(i)
                if options.bRemoveDispatchFields:
                    found = False
                    for dispatch_field in options.dispatch_fields:
                        if EQUAL(dispatch_field, field_defn.GetName()):
                            found = True
                            break
                    if found:
                        continue
                idx = out_lyr.GetLayerDefn().GetFieldIndex(field_defn.GetName())
                if idx >= 0:
                    panMap[i] = idx
                elif out_lyr.CreateField(field_defn) == 0:
                    panMap[i] = out_lyr.GetLayerDefn().GetFieldCount() - 1
            if options.bStyleAsField:
                out_lyr.CreateField(ogr.FieldDefn("OGR_STYLE", ogr.OFTString))
        else:
            panMap = None
        layerMap[out_lyr_name] = [out_lyr, panMap]
    else:
        out_lyr = layerMap[out_lyr_name][0]
        panMap = layerMap[out_lyr_name][1]

    return (out_lyr, panMap)


###############################################################
# convert_layer()


def convert_layer(src_lyr, dst_ds, layerMap, options):

    current_out_lyr = None
    nFeaturesInTransaction = 0

    for feat in src_lyr:

        out_lyr_name = get_out_lyr_name(src_lyr, feat, options)

        geom = feat.GetGeometryRef()
        if geom is not None:
            geom_type = geom.GetGeometryType()
        else:
            geom_type = ogr.wkbUnknown

        (out_lyr, panMap) = get_layer_and_map(
            out_lyr_name, src_lyr, dst_ds, layerMap, geom_type, options
        )

        if options.nGroupTransactions > 0:
            if out_lyr != current_out_lyr:
                if current_out_lyr is not None:
                    current_out_lyr.CommitTransaction()
                current_out_lyr = out_lyr
                current_out_lyr.StartTransaction()
                nFeaturesInTransaction = 0

            if nFeaturesInTransaction == options.nGroupTransactions:
                current_out_lyr.CommitTransaction()
                current_out_lyr.StartTransaction()
        else:
            current_out_lyr = out_lyr

        out_feat = ogr.Feature(out_lyr.GetLayerDefn())
        if panMap is not None:
            out_feat.SetFromWithMap(feat, 1, panMap)
        else:
            out_feat.SetFrom(feat)
        if options.bStyleAsField:
            style = feat.GetStyleString()
            if style is not None:
                out_feat.SetField("OGR_STYLE", style)
        out_lyr.CreateFeature(out_feat)

        nFeaturesInTransaction = nFeaturesInTransaction + 1

    if options.nGroupTransactions > 0 and current_out_lyr is not None:
        current_out_lyr.CommitTransaction()

    return 1


###############################################################
# ogr_dispatch()


def ogr_dispatch(argv, progress=None, progress_arg=None):
    # pylint: disable=unused-argument
    src_filename = None
    dst_filename = None
    driver_name = "ESRI Shapefile"
    options = Options()
    lco = []
    dsco = []
    pszWHERE = None

    if not argv:
        return Usage()

    i = 0
    while i < len(argv):
        arg = argv[i]
        if EQUAL(arg, "-src") and i + 1 < len(argv):
            i = i + 1
            src_filename = argv[i]
        elif EQUAL(arg, "-dst") and i + 1 < len(argv):
            i = i + 1
            dst_filename = argv[i]
        elif EQUAL(arg, "-f") and i + 1 < len(argv):
            i = i + 1
            driver_name = argv[i]

        elif EQUAL(arg, "-a_srs") and i + 1 < len(argv):
            i = i + 1
            pszOutputSRSDef = argv[i]
            if EQUAL(pszOutputSRSDef, "NULL") or EQUAL(pszOutputSRSDef, "NONE"):
                options.bNullifyOutputSRS = True
            else:
                options.poOutputSRS = osr.SpatialReference()
                if options.poOutputSRS.SetFromUserInput(pszOutputSRSDef) != 0:
                    print("Failed to process SRS definition: %s" % pszOutputSRSDef)
                    return 1
        elif EQUAL(arg, "-dsco") and i + 1 < len(argv):
            i = i + 1
            dsco.append(argv[i])
        elif EQUAL(arg, "-lco") and i + 1 < len(argv):
            i = i + 1
            lco.append(argv[i])
        elif EQUAL(arg, "-field") and i + 1 < len(argv):
            i = i + 1
            options.dispatch_fields.append(argv[i])
        elif EQUAL(arg, "-25D_as_2D"):
            options.b25DAs2D = True
        elif EQUAL(arg, "-multi_as_single"):
            options.bMultiAsSingle = True
        elif EQUAL(arg, "-remove_dispatch_fields"):
            options.bRemoveDispatchFields = True
        elif EQUAL(arg, "-prefix_with_layer_name"):
            options.bPrefixWithLayerName = True
        elif EQUAL(arg, "-style_as_field"):
            options.bStyleAsField = True
        elif (EQUAL(arg, "-tg") or EQUAL(arg, "-gt")) and i + 1 < len(argv):
            i = i + 1
            options.nGroupTransactions = int(argv[i])
        elif EQUAL(arg, "-where") and i + 1 < len(argv):
            i = i + 1
            pszWHERE = argv[i]
        elif EQUAL(arg, "-quiet"):
            options.bQuiet = True
        else:
            print("Unrecognized argument : %s" % arg)
            return Usage()
        i = i + 1

    if src_filename is None:
        print("Missing -src")
        return 1

    if dst_filename is None:
        print("Missing -dst")
        return 1

    if not options.dispatch_fields:
        print("Missing -dispatch_field")
        return 1

    src_ds = ogr.Open(src_filename)
    if src_ds is None:
        print("Cannot open source datasource %s" % src_filename)
        return 1

    dst_ds = ogr.Open(dst_filename, update=1)
    if dst_ds is not None:
        if dsco:
            print("-dsco should not be specified for an existing datasource")
            return 1
    else:
        dst_ds = ogr.GetDriverByName(driver_name).CreateDataSource(
            dst_filename, options=dsco
        )
    if dst_ds is None:
        print("Cannot open or create target datasource %s" % dst_filename)
        return 1

    layerMap = {}

    for src_lyr in src_ds:
        if pszWHERE is not None:
            src_lyr.SetAttributeFilter(pszWHERE)
        ret = convert_layer(src_lyr, dst_ds, layerMap, options)
        if ret != 0:
            return ret

    return 0


def main(argv=sys.argv):
    argv = ogr.GeneralCmdLineProcessor(argv)
    return ogr_dispatch(argv[1:])


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
###############################################################################
#
# Project:  GDAL Python samples
# Purpose:  Script to produce a shaded relief image from elevation data
# Author:   Andrey Kiselev, dron@remotesensing.org
#
###############################################################################
# Copyright (c) 2003, Andrey Kiselev <dron@remotesensing.org>
# Copyright (c) 2009, Even Rouault <even dot rouault at spatialys.com>
#
# SPDX-License-Identifier: MIT
###############################################################################

import math
import sys

import numpy as np

from osgeo import gdal, gdal_array

gdal.TermProgress = gdal.TermProgress_nocb


# =============================================================================


def Usage():
    print("Usage: rel.py -lsrcaz azimuth -lsrcel elevation [-elstep step]")
    print("       [-dx xsize] [-dy ysize] [-b band] [-ot type] infile outfile")
    print("Produce a shaded relief image from elevation data")
    print("")
    print(
        "  -lsrcaz azimuth   Azimuth angle of the diffuse light source (0..360 degrees)"
    )
    print(
        "  -lsrcel elevation Elevation angle of the diffuse light source (0..180 degrees)"
    )
    print(
        "  -elstep step      Elevation change corresponding to a change of one grey level"
    )
    print("                    (default 1)")
    print(
        "  -dx xsize         X and Y dimensions (in meters) of one pixel on the ground"
    )
    print("  -dy ysize         (taken from the geotransform matrix by default)")
    print("  -r range	       Dynamic range for output image (default 255)")
    print("  -b band	       Select a band number to convert (default 1)")
    print("  -ot type	       Data type of the output dataset")
    print("                    (Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/")
    print("                     CInt16/CInt32/CFloat32/CFloat64, default is Byte)")
    print("  infile	       Name of the input file")
    print("  outfile	       Name of the output file")
    print("")
    return 2


def ParseType(typ):
    if typ == "Byte":
        return gdal.GDT_Byte
    if typ == "Int16":
        return gdal.GDT_Int16
    if typ == "UInt16":
        return gdal.GDT_UInt16
    if typ == "Int32":
        return gdal.GDT_Int32
    if typ == "UInt32":
        return gdal.GDT_UInt32
    if typ == "Float16":
        return gdal.GDT_Float16
    if typ == "Float32":
        return gdal.GDT_Float32
    if typ == "Float64":
        return gdal.GDT_Float64
    if typ == "CInt16":
        return gdal.GDT_CInt16
    if typ == "CInt32":
        return gdal.GDT_CInt32
    if typ == "CFloat16":
        return gdal.GDT_CFloat16
    if typ == "CFloat32":
        return gdal.GDT_CFloat32
    if typ == "CFloat64":
        return gdal.GDT_CFloat64
    return gdal.GDT_Byte


def main(argv=sys.argv):
    infile = None
    outfile = None
    iBand = 1  # The first band will be converted by default
    driver_name = "GTiff"
    typ = gdal.GDT_Byte

    lsrcaz = None
    lsrcel = None
    elstep = 1.0
    xsize = None
    ysize = None
    dyn_range = 255.0

    # Parse command line arguments.
    i = 1
    while i < len(argv):
        arg = argv[i]

        if arg == "-b":
            i += 1
            iBand = int(argv[i])

        elif arg == "-ot":
            i += 1
            typ = ParseType(argv[i])

        elif arg == "-lsrcaz":
            i += 1
            lsrcaz = float(argv[i])

        elif arg == "-lsrcel":
            i += 1
            lsrcel = float(argv[i])

        elif arg == "-elstep":
            i += 1
            elstep = float(argv[i])

        elif arg == "-dx":
            i += 1
            xsize = float(argv[i])

        elif arg == "-dy":
            i += 1
            ysize = float(argv[i])

        elif arg == "-r":
            i += 1
            dyn_range = float(argv[i])

        elif infile is None:
            infile = arg

        elif outfile is None:
            outfile = arg

        else:
            return Usage()

        i += 1

    if infile is None:
        return Usage()
    if outfile is None:
        return Usage()
    if lsrcaz is None:
        return Usage()
    if lsrcel is None:
        return Usage()

    # translate angles from degrees to radians
    lsrcaz = lsrcaz / 180.0 * math.pi
    lsrcel = lsrcel / 180.0 * math.pi

    lx = -math.sin(lsrcaz) * math.cos(lsrcel)
    ly = math.cos(lsrcaz) * math.cos(lsrcel)
    lz = math.sin(lsrcel)
    lxyz = math.sqrt(lx**2 + ly**2 + lz**2)

    indataset = gdal.Open(infile, gdal.GA_ReadOnly)
    if indataset is None:
        print("Cannot open", infile)
        return 2

    if indataset.RasterXSize < 3 or indataset.RasterYSize < 3:
        print("Input image is too small to process, minimum size is 3x3")
        return 3

    out_driver = gdal.GetDriverByName(driver_name)
    outdataset = out_driver.Create(
        outfile,
        indataset.RasterXSize,
        indataset.RasterYSize,
        indataset.RasterCount,
        typ,
    )
    outband = outdataset.GetRasterBand(1)

    geotransform = indataset.GetGeoTransform()
    projection = indataset.GetProjection()

    if xsize is None:
        xsize = abs(geotransform[1])
    if ysize is None:
        ysize = abs(geotransform[5])

    inband = indataset.GetRasterBand(iBand)
    if inband is None:
        print("Cannot load band", iBand, "from the", infile)
        return 2

    numtype = gdal_array.GDALTypeCodeTonpTypeCode(typ)
    outline = np.empty((1, inband.XSize), numtype)

    prev = inband.ReadAsArray(0, 0, inband.XSize, 1, inband.XSize, 1)[0]
    outband.WriteArray(outline, 0, 0)
    gdal.TermProgress(0.0)

    cur = inband.ReadAsArray(0, 1, inband.XSize, 1, inband.XSize, 1)[0]
    outband.WriteArray(outline, 0, inband.YSize - 1)
    gdal.TermProgress(1.0 / inband.YSize)

    dx = 2 * xsize
    dy = 2 * ysize

    for i in range(1, inband.YSize - 1):
        next_ = inband.ReadAsArray(0, i + 1, inband.XSize, 1, inband.XSize, 1)[0]
        dzx = (cur[0:-2] - cur[2:]) * elstep
        dzy = (prev[1:-1] - next_[1:-1]) * elstep
        nx = -dy * dzx
        ny = dx * dzy
        nz = dx * dy
        nxyz = nx * nx + ny * ny + nz * nz
        nlxyz = nx * lx + ny * ly + nz * lz
        cosine = dyn_range * (nlxyz / (lxyz * np.sqrt(nxyz)))
        cosine = np.clip(cosine, 0.0, dyn_range)
        outline[0, 1:-1] = cosine.astype(numtype)
        outband.WriteArray(outline, 0, i)

        prev = cur
        cur = next_

        # Display progress report on terminal
        gdal.TermProgress(float(i + 1) / (inband.YSize - 1))

    outdataset.SetGeoTransform(geotransform)
    outdataset.SetProjection(projection)

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# ******************************************************************************
#  $Id: loslas2ntv2.py 19102 2010-03-15 23:41:44Z warmerdam $
#
#  Project:  LOS/LAS to NTv2 Translator
#  Purpose:  Translate one or many LOS/LAS sets into an NTv2 datum shift grid
#            file.
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#  Financial Support: i-cubed (http://www.i-cubed.com)
#
# ******************************************************************************
#  Copyright (c) 2010, Frank Warmerdam
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import sys

from osgeo import gdal

# dummy object to hold options


class Options:
    def __init__(self):
        self.verbose_flag = 0
        self.append = 0
        self.create_self = []
        self.metadata = []
        self.negate = 0


# =============================================================================


def Usage():
    print("Usage: loslas2ntv2.py  [-a] [-auto] [-sub_name name] [-parent name]")
    print("                       [-created date] [-updated date] [-gs_type name]")
    print("                       [-system_f name] [-system_t name] [-version version]")
    print("                       [-major_f axis] [-minor_f axis]")
    print("                       [-major_t axis] [-minor_t axis]")
    print("                       [-negate] [src_file.los]... dst_file.gsb ")
    print("")
    print(" -a: append to existing NTv2 file.")
    print(" -auto: process a whole directory of nad27/hpgn los/las files.")
    print(" -negate: reverse direction of change.")
    print("")
    print("eg.")
    print(" loslas2ntv2.py -auto *.los")
    print(
        " loslas2ntv2.py -system_f NAD27 -system_t NAD83 -sub_name conus conus.los conus.gsb"
    )
    return 2


# =============================================================================


def TranslateLOSLAS(los, ntv2_filename, options):

    # Open the LOS and LAS files.
    los_filename = los[:-4] + ".los"
    las_filename = los[:-4] + ".las"

    los_db = gdal.Open(los_filename)
    las_db = gdal.Open(las_filename)

    # Create (or append to) the NTv2 file.
    create_options = options.create_options
    if options.append == 1:
        create_options.append("APPEND_SUBDATASET=YES")

    ntv2_driver = gdal.GetDriverByName("NTv2")
    ntv2_db = ntv2_driver.Create(
        ntv2_filename,
        los_db.RasterXSize,
        los_db.RasterYSize,
        4,
        gdal.GDT_Float32,
        create_options,
    )

    # Copy georeferencing
    ntv2_db.SetGeoTransform(los_db.GetGeoTransform())

    # Copy offsets.
    data = las_db.ReadAsArray()
    if options.negate:
        data = -1 * data
    ntv2_db.GetRasterBand(1).WriteArray(data)

    data = los_db.ReadAsArray()
    if options.negate:
        data = -1 * data
    ntv2_db.GetRasterBand(2).WriteArray(data)

    if options.metadata:
        ntv2_db.SetMetadata(options.metadata)


# =============================================================================
# Auto-process the normal NOAA director of los/las files, producing a
# NAD27 NTv2 file, and an HPGN NTv2 file.


def auto_noaa(options, loslas_list):

    options.append = 0
    options.verbose_flag = 0

    original_metadata = options.metadata

    have_nad27 = 0
    # have_hpgn = 0

    for los in loslas_list:

        options.create_options = []
        options.metadata = original_metadata
        if los.find("hpgn") != -1:
            ntv2_filename = los[:-4] + ".gsb"
            options.append = 0
            options.negate = 1
            options.metadata.append("SUB_NAME=" + los[:2])
            options.metadata.append("MAJOR_F=6378137.0")
            options.metadata.append("MINOR_F=6356752.31414")
            options.metadata.append("MAJOR_T=6378137.0")
            options.metadata.append("MINOR_T=6356752.31414")
            options.metadata.append("SYSTEM_F=HARN")
            options.metadata.append("SYSTEM_T=NAD83")

        else:
            ntv2_filename = "nad27_usa.gsb"
            options.metadata.append("SUB_NAME=" + los[:-4])
            if have_nad27 == 0:
                options.append = 0
                options.metadata.append("MAJOR_F=6378206.4")
                options.metadata.append("MINOR_F=6356583.8")
                options.metadata.append("MAJOR_T=6378137.0")
                options.metadata.append("MINOR_T=6356752.31414")
                options.metadata.append("SYSTEM_F=NAD27")
                options.metadata.append("SYSTEM_T=NAD83")
            else:
                options.append = 1
            have_nad27 = 1

        print("Integrate %s into %s." % (los, ntv2_filename))

        TranslateLOSLAS(los, ntv2_filename, options)

    return 0


def main(argv=sys.argv):
    ntv2_filename = None
    loslas_list = []
    auto_flag = 0

    options = Options()

    argv = gdal.GeneralCmdLineProcessor(argv)
    if argv is None:
        return 0

    # Parse command line arguments.

    i = 1
    while i < len(argv):
        arg = argv[i]

        if arg == "-v":
            options.verbose_flag = 1

        elif arg == "-version" and i < len(argv) - 1:
            options.metadata.append("VERSION=" + argv[i + 1])
            i = i + 1

        elif arg == "-created" and i < len(argv) - 1:
            options.metadata.append("CREATED=" + argv[i + 1])
            i = i + 1

        elif arg == "-updated" and i < len(argv) - 1:
            options.metadata.append("UPDATED=" + argv[i + 1])
            i = i + 1

        elif arg == "-system_f" and i < len(argv) - 1:
            options.metadata.append("SYSTEM_F=" + argv[i + 1])
            i = i + 1

        elif arg == "-system_t" and i < len(argv) - 1:
            options.metadata.append("SYSTEM_T=" + argv[i + 1])
            i = i + 1

        elif arg == "-parent" and i < len(argv) - 1:
            options.metadata.append("PARENT=" + argv[i + 1])
            i = i + 1

        elif arg == "-sub_name" and i < len(argv) - 1:
            options.metadata.append("SUB_NAME=" + argv[i + 1])
            i = i + 1

        elif arg == "-gs_type" and i < len(argv) - 1:
            options.metadata.append("GS_TYPE=" + argv[i + 1])
            i = i + 1

        elif arg == "-major_f" and i < len(argv) - 1:
            options.metadata.append("MAJOR_F=" + argv[i + 1])
            i = i + 1

        elif arg == "-minor_f" and i < len(argv) - 1:
            options.metadata.append("MINOR_F=" + argv[i + 1])
            i = i + 1

        elif arg == "-major_t" and i < len(argv) - 1:
            options.metadata.append("MAJOR_T=" + argv[i + 1])
            i = i + 1

        elif arg == "-minor_t" and i < len(argv) - 1:
            options.metadata.append("MINOR_T=" + argv[i + 1])
            i = i + 1

        elif arg == "-negate":
            options.negate = 1

        elif arg == "-auto":
            auto_flag = 1

        elif arg == "-a":
            options.append = 1

        elif arg[0] == "-":
            return Usage()

        elif arg[-4:] == ".los" or arg[-4:] == ".las":
            loslas_list.append(arg)

        elif arg[-4:] == ".gsb" and ntv2_filename is None:
            ntv2_filename = arg

        else:
            print("Unrecognized argument: ", arg)
            return Usage()

        i = i + 1

    if not loslas_list:
        print("No .los/.las files specified as input.")
        return Usage()

    if auto_flag == 1:
        auto_noaa(options, loslas_list)

    if ntv2_filename is None:
        print("No NTv2 file specified.")
        return Usage()

    # Process loslas files.

    for los in loslas_list:

        TranslateLOSLAS(los, ntv2_filename, options)
        options.append = 1

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# ******************************************************************************
#
#  Name:     gdalimport
#  Project:  GDAL Python Interface
#  Purpose:  Import a GDAL supported file to Tiled GeoTIFF, and build overviews
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#
# ******************************************************************************
#  Copyright (c) 2000, Frank Warmerdam
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import os.path
import sys

from osgeo import gdal


def progress_cb(complete, message, cb_data):
    print("%s %d" % (cb_data, complete))


def main(argv=sys.argv):
    argv = gdal.GeneralCmdLineProcessor(argv)
    if argv is None:
        return 0

    if len(argv) < 2:
        print("Usage: gdalimport.py [--help-general] source_file [newfile]")
        return 2

    filename = argv[1]
    dataset = gdal.Open(filename)
    if dataset is None:
        print("Unable to open %s" % filename)
        return 1

    geotiff = gdal.GetDriverByName("GTiff")
    if geotiff is None:
        print("GeoTIFF driver not registered.")
        return 1

    if len(argv) < 3:
        newbase, ext = os.path.splitext(os.path.basename(filename))
        newfile = newbase + ".tif"
        i = 0
        while os.path.isfile(newfile):
            i = i + 1
            newfile = newbase + "_" + str(i) + ".tif"
    else:
        newfile = argv[2]

    print("Importing to Tiled GeoTIFF file: %s" % newfile)
    new_dataset = geotiff.CreateCopy(
        newfile,
        dataset,
        0,
        [
            "TILED=YES",
        ],
        callback=progress_cb,
        callback_data="Translate: ",
    )
    dataset = None

    print("Building overviews")
    new_dataset.BuildOverviews(
        "average", callback=progress_cb, callback_data="Overviews: "
    )
    new_dataset = None

    print("Done")


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL utils.auxiliary
#  Purpose:  utility functions for gdal-numpy integration
#  Author:   Idan Miara <idan@miara.com>
#
# ******************************************************************************
#  Copyright (c) 2020, Idan Miara <idan@miara.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import numpy as np

from osgeo import gdal, gdal_array
from osgeo_utils.auxiliary.array_util import ArrayOrScalarLike, ScalarLike


def GDALTypeCodeToNumericTypeCodeEx(buf_type, signed_byte, default=None):
    typecode = gdal_array.GDALTypeCodeToNumericTypeCode(buf_type)
    if typecode is None:
        typecode = default

    if buf_type == gdal.GDT_Byte and signed_byte:
        typecode = np.int8
    return typecode


def GDALTypeCodeAndNumericTypeCodeFromDataSet(ds):
    band = ds.GetRasterBand(1)
    buf_type = band.DataType
    signed_byte = False
    if buf_type == gdal.GDT_Byte:
        band._EnablePixelTypeSignedByteWarning(False)
        signed_byte = (
            band.GetMetadataItem("PIXELTYPE", "IMAGE_STRUCTURE") == "SIGNEDBYTE"
        )
        band._EnablePixelTypeSignedByteWarning(True)
    np_typecode = GDALTypeCodeToNumericTypeCodeEx(
        buf_type, signed_byte=signed_byte, default=np.float32
    )
    return buf_type, np_typecode


def array_dist(
    x: ArrayOrScalarLike, y: ArrayOrScalarLike, is_max: bool = True
) -> ScalarLike:
    if isinstance(x, ScalarLike.__args__):
        return abs(x - y)
    if not isinstance(x, np.ndarray):
        x = np.array(x)
    if not isinstance(y, np.ndarray):
        y = np.array(y)
    diff = np.abs(x - y)
    return np.max(diff) if is_max else np.min(diff)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL utils.auxiliary
#  Purpose:  general use utility functions
#  Author:   Even Rouault <even.rouault at spatialys.com>
#  Author:   Idan Miara <idan@miara.com>
#
# ******************************************************************************
#  Copyright (c) 2015, Even Rouault <even.rouault at spatialys.com>
#  Copyright (c) 2020, Idan Miara <idan@miara.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import os.path
from enum import Enum
from numbers import Number, Real
from pathlib import Path
from typing import List, Optional, Sequence, Tuple, TypeVar, Union

T = TypeVar("T")
MaybeSequence = Union[T, Sequence[T]]
PathLikeOrStr = Union[str, os.PathLike]
SequenceNotString = Union[List, Tuple]
Real2D = Tuple[Real, Real]
OptionalBoolStr = Optional[Union[str, bool]]


def enum_to_str(enum_or_str: Union[Enum, str]) -> str:
    return enum_or_str.name if isinstance(enum_or_str, Enum) else str(enum_or_str)


def is_path_like(s) -> bool:
    return isinstance(s, PathLikeOrStr.__args__)


def get_suffix(filename: PathLikeOrStr) -> str:
    return Path(filename).suffix  # same as os.path.splitext(filename)[1]


def get_extension(filename: PathLikeOrStr) -> str:
    """
    returns the suffix without the leading dot.
    special case for shp.zip and gpkg.zip
    """
    lower_filename = os.fspath(filename).lower()
    if lower_filename.endswith(".shp.zip"):
        return "shp.zip"
    if lower_filename.endswith(".gpkg.zip"):
        return "gpkg.zip"
    ext = get_suffix(filename)
    if ext.startswith("."):
        ext = ext[1:]
    return ext


def get_byte(number: int, i: int):
    """returns the i-th byte from an integer"""
    return (number & (0xFF << (i * 8))) >> (i * 8)


def path_join(*args) -> str:
    return os.path.join(*(str(arg) for arg in args))


def num(s: Union[Number, str]) -> Number:
    if isinstance(s, Number):
        return s
    else:
        try:
            return int(s)
        except ValueError:
            return float(s)


def num_or_none(s: Optional[Union[Number, str]]) -> Optional[Number]:
    try:
        return num(s)
    except Exception:
        return None


def is_true(
    b: OptionalBoolStr,
    accept_none: bool = False,
    case_insensitive=True,
    false_str=("NO", "FALSE", "OFF"),
    true_str=("YES", "TRUE", "ON"),
) -> Optional[bool]:
    """
    Returns a boolean value that is represented by a string or a bool
    correlated to the c++ implementation:
    https://github.com/OSGeo/gdal/blob/362541e961cf8cab046dfbae1bde8ce559b4cb40/gdal/gcore/gdaldriver.cpp#L1973
    """
    if isinstance(b, bool):
        return b
    if not b and accept_none:
        return None
    if isinstance(b, str):
        if case_insensitive:
            b = b.upper()
        if b in false_str:
            return False
        if b in true_str:
            return True
    raise Exception(f"{b} is not accepted as a valid boolean-like value")



#!/usr/bin/env python3
###############################################################################
#
# Project:  GDAL
# Purpose:  Handles progress callback functions
# Author:   Idan Miara <idan@miara.com>
#
###############################################################################
# Copyright (c) 2020, Idan Miara <idan@miara.com>
#
# SPDX-License-Identifier: MIT
###############################################################################
from enum import Enum, auto
from numbers import Real
from typing import Callable, Optional, Union

from osgeo.gdal import TermProgress_nocb


class PredefinedProgressCallback(Enum):
    TermProgress = auto()  # default gdal core term progress
    TermProgressPy = (
        auto()
    )  # use this option to be able to put a breakpoint inside the callback
    TermProgressSimple = auto()  # simple print progress


ProgressCallback = Optional[Callable[[Real], None]]
OptionalProgressCallback = Union[
    type(...), PredefinedProgressCallback, ProgressCallback
]


def simple_term_progress(progress: Real):
    print(str(round(progress * 100)) + "%", end=" ")


def term_progress_from_to(r0: Optional[Real], r1: Real):
    """prints the progress from r0 to r1"""
    i0 = 0 if (r0 is None) or (r0 > r1) else round(r0 * 100) + 1
    i1 = round(r1 * 100) + 1
    for i in range(i0, i1):
        print(str(i) if i % 5 == 0 else ".", end="", flush=True)
    if r1 >= 1:
        print("% done!")


def get_py_term_progress_callback():
    term_progress_py_last = None

    def py_term_progress(progress: Real):
        nonlocal term_progress_py_last

        r0 = term_progress_py_last
        r1 = progress
        term_progress_from_to(r0, r1)
        term_progress_py_last = progress

    return py_term_progress


def get_progress_callback(callback: OptionalProgressCallback) -> ProgressCallback:
    """returns a predefined callback or the given callback"""
    if callback is None:
        return None
    elif callback == PredefinedProgressCallback.TermProgress or callback == ...:
        return TermProgress_nocb
    elif callback == PredefinedProgressCallback.TermProgressPy:
        return get_py_term_progress_callback()
    elif callback == PredefinedProgressCallback.TermProgressSimple:
        return simple_term_progress

    return callback



#!/usr/local/bin/python3.13
###############################################################################
# Project:  GDAL utils
# Purpose:  Get min/max location
# Author:   Even Rouault <even@spatialys.com>
#
###############################################################################
# Copyright (c) 2024, Even Rouault <even@spatialys.com>
#
# SPDX-License-Identifier: MIT
###############################################################################

import sys
import textwrap
from typing import Optional

from osgeo import gdal, osr
from osgeo_utils.auxiliary.gdal_argparse import GDALArgumentParser, GDALScript
from osgeo_utils.auxiliary.util import PathOrDS, open_ds


def gdalminmaxlocation_util(
    filename_or_ds: PathOrDS,
    band_num: int,
    open_options: Optional[dict] = None,
    **kwargs,
):
    ds = open_ds(filename_or_ds, open_options=open_options)
    band = ds.GetRasterBand(band_num)
    ret = band.ComputeMinMaxLocation()
    if ret is None:
        print("No valid pixels")
        return 1
    gt = ds.GetGeoTransform(can_return_null=True)
    if gt:
        srs = ds.GetSpatialRef()
        if srs:
            wgs84 = osr.SpatialReference()
            wgs84.SetFromUserInput("WGS84")
            wgs84.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
            ct = osr.CreateCoordinateTransformation(srs, wgs84)
            georefX, georefY = gdal.ApplyGeoTransform(
                gt, ret.minX + 0.5, ret.minY + 0.5
            )
            long, lat, _ = ct.TransformPoint(georefX, georefY)
            print(
                f"Minimum={ret.min} at (col,line)=({ret.minX},{ret.minY}), (X,Y)_georef=({georefX},{georefY}), (long,lat)_WGS84=({long:.7f},{lat:.7f})"
            )
            georefX, georefY = gdal.ApplyGeoTransform(
                gt, ret.maxX + 0.5, ret.maxY + 0.5
            )
            long, lat, _ = ct.TransformPoint(georefX, georefY)
            print(
                f"Maximum={ret.max} at (col,line)=({ret.maxX},{ret.maxY}), (X,Y)_georef=({georefX},{georefY}), (long,lat)_WGS84=({long:.7f},{lat:.7f})"
            )
        else:
            georefX, georefY = gdal.ApplyGeoTransform(
                gt, ret.minX + 0.5, ret.minY + 0.5
            )
            print(
                f"Minimum={ret.min} at (col,line)=({ret.minX},{ret.minY}), (X,Y)_georef=({georefX},{georefY})"
            )
            georefX, georefY = gdal.ApplyGeoTransform(
                gt, ret.maxX + 0.5, ret.maxY + 0.5
            )
            print(
                f"Maximum={ret.max} at (col,line)=({ret.maxX},{ret.maxY}), (X,Y)_georef=({georefX},{georefY})"
            )
    else:
        print(f"Minimum={ret.min} at (col,line)=({ret.minX},{ret.minY})")
        print(f"Maximum={ret.max} at (col,line)=({ret.maxX},{ret.maxY})")

    return 0


class GDALMinMaxLocation(GDALScript):
    def __init__(self):
        super().__init__()
        self.title = "Raster min/max location query tool"
        self.description = textwrap.dedent(
            """\
            The gdal_minmax_location utility returns the location where min/max values of a raster are hit."""
        )
        self.interactive_mode = None

    def get_parser(self, argv) -> GDALArgumentParser:
        parser = self.parser

        parser.add_argument(
            "-b",
            dest="band_num",
            metavar="band",
            type=int,
            default=1,
            help="Selects a band to query (default: first one).",
        )

        parser.add_argument(
            "-oo",
            dest="open_options",
            metavar="NAME=VALUE",
            help="Dataset open option (format specific).",
            nargs="+",
        )

        parser.add_argument(
            "filename_or_ds",
            metavar="filename",
            type=str,
            help="The source GDAL raster datasource name.",
        )

        return parser

    def augment_kwargs(self, kwargs) -> dict:
        return kwargs

    def doit(self, **kwargs):
        return gdalminmaxlocation_util(**kwargs)


def main(argv=sys.argv):
    gdal.UseExceptions()
    return GDALMinMaxLocation().main(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL
#  Purpose:  rectangle class (x, y, width, height)
#  Author:   Idan Miara, <idan@miara.com>
#
# ******************************************************************************
#  Copyright (c) 2020, Idan Miara <idan@miara.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************


class GeoRectangle:
    __slots__ = ["x", "y", "w", "h"]

    def __init__(self, x, y, w, h, allow_negative_size=False):
        if w <= 0:
            if allow_negative_size:
                x = x + w
                w = -w
            else:
                w = 0
        if h <= 0:
            if allow_negative_size:
                y = y + h
                h = -h
            else:
                h = 0
        self.x = x
        self.y = y
        self.w = w
        self.h = h

    def __eq__(self, other):
        if not isinstance(other, GeoRectangle):
            # don't attempt to compare against unrelated types
            return False
        return self.xywh == other.xywh

    def __round__(self, *args, **kwargs):
        return self.from_lrdu(*(round(i, *args, **kwargs) for i in self.lrdu))

    def is_empty(self):
        return self.w <= 0 or self.h <= 0

    def intersect(self, other: "GeoRectangle"):
        return GeoRectangle.from_min_max(
            max(self.min_x, other.min_x),
            min(self.max_x, other.max_x),
            max(self.min_y, other.min_y),
            min(self.max_y, other.max_y),
        )

    def union(self, other: "GeoRectangle"):
        return GeoRectangle.from_min_max(
            min(self.min_x, other.min_x),
            max(self.max_x, other.max_x),
            min(self.min_y, other.min_y),
            max(self.max_y, other.max_y),
        )

    def round(self, digits):
        self.x = round(self.x, digits)
        self.y = round(self.y, digits)
        self.w = round(self.w, digits)
        self.h = round(self.h, digits)

    def align(self, geo_transform):
        # compute the pixel-aligned bounding box (larger than the feature's bbox)
        left = self.min_x - (self.min_x - geo_transform[0]) % geo_transform[1]
        right = self.max_x + (
            geo_transform[1] - ((self.max_x - geo_transform[0]) % geo_transform[1])
        )
        bottom = self.min_y + (
            geo_transform[5] - ((self.min_y - geo_transform[3]) % geo_transform[5])
        )
        top = self.max_y - (self.max_y - geo_transform[3]) % geo_transform[5]
        return self.from_lrud(left, right, top, bottom)

    def get_partition(self, part: "GeoRectangle"):
        # part: x,y - part indexes; w,h - part counts
        part_width = self.w / part.w
        part_hight = self.h / part.h
        return GeoRectangle(
            self.x + part.x * part_width,
            self.y + part.y * part_hight,
            part_width,
            part_hight,
        )

    @classmethod
    def empty(cls):
        return cls(0, 0, 0, 0)

    @classmethod
    def from_lrud(cls, l, r, u, d):
        ret = cls(l, d, r - l, u - d)
        return ret

    @classmethod
    # same as min_max
    def from_lrdu(cls, l, r, d, u):
        ret = cls(l, d, r - l, u - d)
        return ret

    @classmethod
    def from_lurd(cls, l, u, r, d):
        """from projwin (minx maxy maxx miny) == (ulx uly lrx lry) == (l u r d)"""
        ret = cls(l, d, r - l, u - d)
        return ret

    @classmethod
    # same as min_max
    def from_xwyh(cls, x, w, y, h, allow_negative_size=False):
        ret = cls(x, y, w, h, allow_negative_size)
        return ret

    @classmethod
    # # same as cls
    def from_xywh(cls, x, y, w, h, allow_negative_size=False):
        ret = cls(x, y, w, h, allow_negative_size)
        return ret

    @classmethod
    # # same as cls
    def from_xywhps(cls, x, y, w, h, px, py):
        ret = cls(x, y, w * px, h * py, True)
        return ret

    @classmethod
    # same as lrdu
    def from_min_max(cls, min_x, max_x, min_y, max_y):
        ret = cls(min_x, min_y, max_x - min_x, max_y - min_y)
        return ret

    @classmethod
    def from_center_and_radius(cls, cent_x, cent_y, rad_x, rad_y=None):
        if rad_y is None:
            rad_y = rad_x
        x = cent_x - rad_x
        y = cent_y - rad_y
        w = rad_x * 2
        h = rad_y * 2
        ret = cls(x, y, w, h)
        return ret

    @classmethod
    def from_points(cls, points):
        return cls.from_min_max(
            min(p[0] for p in points),
            max(p[0] for p in points),
            min(p[1] for p in points),
            max(p[1] for p in points),
        )

    @classmethod
    def from_geotransform_and_size(cls, gt, size):
        if gt[2] or gt[4]:
            return cls.from_points(get_points_extent(gt, *size))
        else:
            # faster method
            origin = (gt[0], gt[3])
            pixel_size = (gt[1], gt[5])
            extent = cls.from_xywhps(*origin, *size, *pixel_size)
            # extent_b = cls.from_points(get_points_extent(gt, *size))
            # assert extent == extent_b
            return extent

    def to_pixels(self, pixel_size):
        return self.from_xwyh(
            self.x / pixel_size[0],
            self.w / pixel_size[0],
            self.y / pixel_size[1],
            self.h / pixel_size[1],
            True,
        )

    @classmethod
    def from_geotransform_and_size_to_pix(cls, gt, size):
        origin = (gt[0], gt[3])
        pixel_size = (gt[1], gt[5])
        pix_origin = list(origin[i] / pixel_size[i] for i in (0, 1))
        # pix_bounds = list(origin[i] / pixel_size[i] + size[i] for i in (0, 1))
        return cls.from_xwyh(pix_origin[0], size[0], pix_origin[1], size[1])

    @property
    def area(self):
        return self.w * self.h

    @property
    def size(self):
        return self.w, self.h

    @property
    def left(self):
        return self.x

    @property
    def right(self):
        return self.x + self.w

    @property
    def down(self):
        return self.y

    @property
    def up(self):
        return self.y + self.h

    @property
    def min_x(self):
        return self.x

    @property
    def max_x(self):
        return self.x + self.w

    @property
    def min_y(self):
        return self.y

    @property
    def max_y(self):
        return self.y + self.h

    @property
    def lurd(self):
        return self.left, self.up, self.right, self.down

    @property
    def lrud(self):
        return self.left, self.right, self.up, self.down

    @property
    def ldru(self):
        return self.left, self.down, self.right, self.up

    @property
    def lrdu(self):
        return self.left, self.right, self.down, self.up

    @property
    def xywh(self):
        return self.x, self.y, self.w, self.h

    @property
    def xwyh(self):
        return self.x, self.w, self.y, self.h

    @property
    def min_max(self):
        return self.min_x, self.max_x, self.min_y, self.max_y

    def __str__(self):
        return f"Rectangle(x[{self.min_x},{self.max_x}], y[{self.min_y},{self.max_y}] wh[{self.w},{self.h}])"

    def __repr__(self):
        return f"Rectangle(x:{self.x}, y:{self.y}, w:{self.w}, h:{self.h})"

    def __hash__(self):
        return hash(self.xywh)


def get_points_extent(gt, cols, rows):
    """Return list of corner coordinates from a geotransform"""

    def transform_point(px, py):
        x = gt[0] + (px * gt[1]) + (py * gt[2])
        y = gt[3] + (px * gt[4]) + (py * gt[5])
        return x, y

    return [
        transform_point(0, 0),
        transform_point(0, rows),
        transform_point(cols, rows),
        transform_point(cols, 0),
    ]



#!/usr/local/bin/python3.13
# ******************************************************************************
#
#  Project:  GDAL
#  Purpose:  Application to checksum a GDAL image file.
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#
# ******************************************************************************
#  Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import sys

from osgeo import gdal


def Usage():
    print("Usage: gdalchksum.py [-b band] [-srcwin xoff yoff xsize ysize] file")
    return 2


def main(argv=sys.argv):
    srcwin = None
    bands = []

    filename = None

    argv = gdal.GeneralCmdLineProcessor(argv)
    if argv is None:
        return 0

    # Parse command line arguments.
    i = 1
    while i < len(argv):
        arg = argv[i]

        if arg == "-b":
            i = i + 1
            bands.append(int(argv[i]))

        elif arg == "-srcwin":
            srcwin = [
                int(argv[i + 1]),
                int(argv[i + 2]),
                int(argv[i + 3]),
                int(argv[i + 3]),
            ]
            i = i + 4

        elif filename is None:
            filename = argv[i]

        else:
            return Usage()

        i = i + 1

    if filename is None:
        return Usage()

    # Open source file

    ds = gdal.Open(filename)
    if ds is None:
        print("Unable to open %s" % filename)
        return 1

    # Default values

    if srcwin is None:
        srcwin = [0, 0, ds.RasterXSize, ds.RasterYSize]

    if not bands:
        bands = list(range(1, (ds.RasterCount + 1)))

    # Generate checksums

    for band_num in bands:
        oBand = ds.GetRasterBand(band_num)
        result = oBand.Checksum(srcwin[0], srcwin[1], srcwin[2], srcwin[3])
        print(result)

    ds = None


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL Python Interface
#  Purpose:  Application for converting raster data to a vector polygon layer.
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#
# ******************************************************************************
#  Copyright (c) 2008, Frank Warmerdam
#  Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
#  Copyright (c) 2021, Idan Miara <idan@miara.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import sys
import textwrap
from typing import Optional, Union

from osgeo import gdal, ogr
from osgeo_utils.auxiliary.gdal_argparse import GDALArgumentParser, GDALScript
from osgeo_utils.auxiliary.util import GetOutputDriverFor, enable_gdal_exceptions


@enable_gdal_exceptions
def gdal_polygonize(
    src_filename: Optional[str] = None,
    band_number: Union[int, str] = 1,
    dst_filename: Optional[str] = None,
    overwrite: bool = False,
    driver_name: Optional[str] = None,
    dst_layername: Optional[str] = None,
    dst_fieldname: Optional[str] = None,
    quiet: bool = False,
    mask: str = "default",
    options: Optional[list] = None,
    layer_creation_options: Optional[list] = None,
    connectedness8: bool = False,
):

    if isinstance(band_number, str) and not band_number.startswith("mask"):
        band_number = int(band_number)

    options = options or []

    if connectedness8:
        options.append("8CONNECTED=8")

    if driver_name is None:
        driver_name = GetOutputDriverFor(dst_filename, is_raster=False)

    if dst_layername is None:
        dst_layername = "out"

    # =============================================================================
    # Open source file
    # =============================================================================

    src_ds = gdal.Open(src_filename)

    if src_ds is None:
        print("Unable to open %s" % src_filename)
        return 1

    if band_number == "mask":
        srcband = src_ds.GetRasterBand(1).GetMaskBand()
        # Workaround the fact that most source bands have no dataset attached
        options.append("DATASET_FOR_GEOREF=" + src_filename)
    elif isinstance(band_number, str) and band_number.startswith("mask,"):
        srcband = src_ds.GetRasterBand(int(band_number[len("mask,") :])).GetMaskBand()
        # Workaround the fact that most source bands have no dataset attached
        options.append("DATASET_FOR_GEOREF=" + src_filename)
    else:
        srcband = src_ds.GetRasterBand(band_number)

    if mask == "default":
        maskband = srcband.GetMaskBand()
    elif mask == "none":
        maskband = None
    else:
        mask_ds = gdal.Open(mask)
        maskband = mask_ds.GetRasterBand(1)

    # =============================================================================
    #       Try opening the destination file as an existing file.
    # =============================================================================

    try:
        gdal.PushErrorHandler("CPLQuietErrorHandler")
        dst_ds = ogr.Open(dst_filename, update=1)
        gdal.PopErrorHandler()
    except Exception:
        try:
            dst_ds = ogr.Open(dst_filename)
        except Exception:
            dst_ds = None
        if dst_ds and not overwrite:
            raise Exception(
                f"{dst_filename} exists, but cannot be updated. You may need to remove it before or use -overwrite"
            )

    if dst_ds is not None and overwrite:
        cnt = dst_ds.GetLayerCount()
        iLayer = None  # initialize in case there are no loop iterations
        for iLayer in range(cnt):
            poLayer = dst_ds.GetLayer(iLayer)
            if poLayer is not None and poLayer.GetName() == dst_layername:
                break

        delete_ok = False
        if iLayer != cnt:
            if dst_ds.TestCapability(ogr.ODsCDeleteLayer) == 1:
                try:
                    delete_ok = dst_ds.DeleteLayer(iLayer) == ogr.OGRERR_NONE
                except Exception:
                    delete_ok = False

        if not delete_ok:
            if cnt == 1:
                dst_ds = None
                if gdal.VSIStatL(dst_filename):
                    gdal.Unlink(dst_filename)

    # =============================================================================
    # 	Create output file.
    # =============================================================================
    if dst_ds is None:
        drv = ogr.GetDriverByName(driver_name)
        if not quiet:
            print("Creating output %s of format %s." % (dst_filename, driver_name))
        dst_ds = drv.CreateDataSource(dst_filename)
        if dst_ds is None:
            print('Cannot create datasource "%s"' % dst_filename)
            return 1

    # =============================================================================
    #       Find or create destination layer.
    # =============================================================================
    try:
        dst_layer = dst_ds.GetLayerByName(dst_layername)
    except Exception:
        dst_layer = None

    dst_field: int = -1
    if dst_layer is None:

        srs = src_ds.GetSpatialRef()
        dst_layer = dst_ds.CreateLayer(
            dst_layername,
            geom_type=ogr.wkbPolygon,
            srs=srs,
            options=layer_creation_options if layer_creation_options else [],
        )

        if dst_fieldname is None:
            dst_fieldname = "DN"

        data_type = ogr.OFTInteger
        if srcband.DataType == gdal.GDT_Int64 or srcband.DataType == gdal.GDT_UInt64:
            data_type = ogr.OFTInteger64

        fd = ogr.FieldDefn(dst_fieldname, data_type)
        dst_layer.CreateField(fd)
        dst_field = 0
    else:
        if layer_creation_options:
            print(
                "Warning: layer_creation_options will be ignored as the layer already exists"
            )

        if dst_fieldname is not None:
            dst_field = dst_layer.GetLayerDefn().GetFieldIndex(dst_fieldname)
            if dst_field < 0:
                print(
                    "Warning: cannot find field '%s' in layer '%s'"
                    % (dst_fieldname, dst_layername)
                )

    # =============================================================================
    # Invoke algorithm.
    # =============================================================================

    if quiet:
        prog_func = None
    else:
        prog_func = gdal.TermProgress_nocb

    dst_layer.StartTransaction()
    result = gdal.Polygonize(
        srcband, maskband, dst_layer, dst_field, options, callback=prog_func
    )
    if result == gdal.CE_None:
        dst_layer.CommitTransaction()
    else:
        dst_layer.RollbackTransaction()

    srcband = None
    src_ds = None
    dst_ds = None
    mask_ds = None

    return result


class GDALPolygonize(GDALScript):
    def __init__(self):
        super().__init__()
        self.title = "Produces a polygon feature layer from a raster"
        self.description = textwrap.dedent(
            """\
            This utility creates vector polygons for all connected regions of pixels in the raster
            sharing a common pixel value. Each polygon is created with an attribute indicating
            the pixel value of that polygon.
            A raster mask may also be provided to determine which pixels are eligible for processing.
            The utility will create the output vector datasource if it does not already exist,
            otherwise it will try to append to an existing one.
            The utility is based on the GDALPolygonize() function
            which has additional details on the algorithm."""
        )

    def get_parser(self, argv) -> GDALArgumentParser:
        parser = self.parser

        parser.add_argument(
            "-q",
            "-quiet",
            dest="quiet",
            action="store_true",
            help="The script runs in quiet mode. "
            "The progress monitor is suppressed and routine messages are not displayed.",
        )

        parser.add_argument(
            "-8",
            dest="connectedness8",
            action="store_true",
            help="Use 8 connectedness. Default is 4 connectedness.",
        )

        parser.add_argument(
            "-o",
            dest="options",
            type=str,
            action="append",
            metavar="name=value",
            help="Specify a special argument to the algorithm. This may be specified multiple times.",
        )

        parser.add_argument(
            "-mask",
            dest="mask",
            type=str,
            metavar="filename",
            default="default",
            help="Use the first band of the specified file as a validity mask "
            "(zero is invalid, non-zero is valid).",
        )

        parser.add_argument(
            "-nomask",
            dest="mask",
            action="store_const",
            const="none",
            default="default",
            help="Do not use the default validity mask for the input band "
            "(such as nodata, or alpha masks).",
        )

        parser.add_argument(
            "-b",
            "-band",
            dest="band_number",
            metavar="band",
            type=str,
            default="1",
            help="The band on <raster_file> to build the polygons from. "
            'The value can also be set to "mask", '
            "to indicate that the mask band of the first band must be used "
            '(or "mask,band_number" for the mask of a specified band).',
        )

        parser.add_argument(
            "-of",
            "-f",
            dest="driver_name",
            metavar="ogr_format",
            help="Select the output format. "
            "if not specified, the format is guessed from the extension. "
            "Use the short format name.",
        )

        parser.add_argument(
            "-lco",
            dest="layer_creation_options",
            type=str,
            action="append",
            metavar="name=value",
            help="Specify a layer creation option. This may be specified multiple times.",
        )

        parser.add_argument(
            "-overwrite",
            dest="overwrite",
            action="store_true",
            help="overwrite output file if it already exists",
        )

        parser.add_argument(
            "src_filename",
            type=str,
            help="The source raster file from which polygons are derived.",
        )

        parser.add_argument(
            "dst_filename",
            type=str,
            help="The destination vector file to which the polygons will be written.",
        )

        parser.add_argument(
            "dst_layername",
            type=str,
            nargs="?",
            help="The name of the layer created to hold the polygon features.",
        )

        parser.add_argument(
            "dst_fieldname",
            type=str,
            nargs="?",
            help='The name of the field to create (defaults to "DN").',
        )

        return parser

    def doit(self, **kwargs):
        return gdal_polygonize(**kwargs)


def main(argv=sys.argv):
    return GDALPolygonize().main(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13

import sys

from osgeo.gdal import deprecation_warn

# import osgeo_utils.ogr_layer_algebra as a convenience to use as a script
from osgeo_utils.ogr_layer_algebra import *  # noqa
from osgeo_utils.ogr_layer_algebra import main

deprecation_warn("ogr_layer_algebra")
sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# *****************************************************************************
#
# Project:  GDAL Utilities
# Purpose:  Python port of Commandline application to list info about a file.
# Author:   Even Rouault, <even dot rouault at spatialys.com>
#
# Port from gdalinfo.c whose author is Frank Warmerdam
#
# *****************************************************************************
# Copyright (c) 2010-2011, Even Rouault <even dot rouault at spatialys.com>
# Copyright (c) 1998, Frank Warmerdam
#
# SPDX-License-Identifier: MIT
# ***************************************************************************/

import math
import sys

from osgeo import gdal, osr

# **********************************************************************
#                               Usage()
# **********************************************************************


def Usage():
    print(
        "Usage: gdalinfo [--help-general] [-mm] [-stats] [-hist] [-nogcp] [-nomd]\n"
        + "                [-norat] [-noct] [-nofl] [-checksum] [-mdd domain]... datasetname"
    )
    return 2


def EQUAL(a, b):
    return a.lower() == b.lower()


def main(argv=None):
    bComputeMinMax = False
    bShowGCPs = True
    bShowMetadata = True
    bShowRAT = True
    bStats = False
    bApproxStats = True
    bShowColorTable = True
    bComputeChecksum = False
    bReportHistograms = False
    pszFilename = None
    papszExtraMDDomains = []
    pszProjection = None
    hTransform = None
    bShowFileList = True

    # Must process GDAL_SKIP before GDALAllRegister(), but we can't call
    # GDALGeneralCmdLineProcessor before it needs the drivers to be registered
    # for the --format or --formats options
    # for( i = 1; i < argc; i++ )
    # {
    #    if EQUAL(argv[i],"--config") and i + 2 < argc and EQUAL(argv[i + 1], "GDAL_SKIP"):
    #    {
    #        CPLSetConfigOption( argv[i+1], argv[i+2] );
    #
    #        i += 2;
    #    }
    # }
    #
    # GDALAllRegister();

    if argv is None:
        argv = sys.argv

    argv = gdal.GeneralCmdLineProcessor(argv)

    if argv is None:
        return 1

    nArgc = len(argv)
    # --------------------------------------------------------------------
    #      Parse arguments.
    # --------------------------------------------------------------------
    i = 1
    while i < nArgc:

        if EQUAL(argv[i], "--utility_version"):
            print(
                "%s is running against GDAL %s"
                % (argv[0], gdal.VersionInfo("RELEASE_NAME"))
            )
            return 0
        elif EQUAL(argv[i], "-mm"):
            bComputeMinMax = True
        elif EQUAL(argv[i], "-hist"):
            bReportHistograms = True
        elif EQUAL(argv[i], "-stats"):
            bStats = True
            bApproxStats = False
        elif EQUAL(argv[i], "-approx_stats"):
            bStats = True
            bApproxStats = True
        elif EQUAL(argv[i], "-checksum"):
            bComputeChecksum = True
        elif EQUAL(argv[i], "-nogcp"):
            bShowGCPs = False
        elif EQUAL(argv[i], "-nomd"):
            bShowMetadata = False
        elif EQUAL(argv[i], "-norat"):
            bShowRAT = False
        elif EQUAL(argv[i], "-noct"):
            bShowColorTable = False
        elif EQUAL(argv[i], "-mdd") and i < nArgc - 1:
            i = i + 1
            papszExtraMDDomains.append(argv[i])
        elif EQUAL(argv[i], "-nofl"):
            bShowFileList = False
        elif argv[i][0] == "-":
            return Usage()
        elif pszFilename is None:
            pszFilename = argv[i]
        else:
            return Usage()

        i = i + 1

    if pszFilename is None:
        return Usage()

    # --------------------------------------------------------------------
    #      Open dataset.
    # --------------------------------------------------------------------
    hDataset = gdal.Open(pszFilename, gdal.GA_ReadOnly)

    if hDataset is None:

        print("gdalinfo failed - unable to open '%s'." % pszFilename)

        return 1

    # --------------------------------------------------------------------
    #      Report general info.
    # --------------------------------------------------------------------
    hDriver = hDataset.GetDriver()
    print("Driver: %s/%s" % (hDriver.ShortName, hDriver.LongName))

    papszFileList = hDataset.GetFileList()
    if papszFileList is None or not papszFileList:
        print("Files: none associated")
    else:
        print("Files: %s" % papszFileList[0])
        if bShowFileList:
            for i in range(1, len(papszFileList)):
                print("       %s" % papszFileList[i])

    print("Size is %d, %d" % (hDataset.RasterXSize, hDataset.RasterYSize))

    # --------------------------------------------------------------------
    #      Report projection.
    # --------------------------------------------------------------------
    pszProjection = hDataset.GetProjectionRef()
    if pszProjection is not None:

        hSRS = osr.SpatialReference()
        with gdal.quiet_errors(), osr.ExceptionMgr(useExceptions=False):
            if hSRS.ImportFromWkt(pszProjection) == gdal.CE_None:
                pszPrettyWkt = hSRS.ExportToPrettyWkt(False)

                print("Coordinate System is:\n%s" % pszPrettyWkt)
            else:
                print("Coordinate System is `%s'" % pszProjection)

    # --------------------------------------------------------------------
    #      Report Geotransform.
    # --------------------------------------------------------------------
    adfGeoTransform = hDataset.GetGeoTransform(can_return_null=True)
    if adfGeoTransform is not None:

        if adfGeoTransform[2] == 0.0 and adfGeoTransform[4] == 0.0:
            print("Origin = (%.15f,%.15f)" % (adfGeoTransform[0], adfGeoTransform[3]))

            print(
                "Pixel Size = (%.15f,%.15f)" % (adfGeoTransform[1], adfGeoTransform[5])
            )

        else:
            print(
                "GeoTransform =\n"
                "  %.16g, %.16g, %.16g\n"
                "  %.16g, %.16g, %.16g"
                % (
                    adfGeoTransform[0],
                    adfGeoTransform[1],
                    adfGeoTransform[2],
                    adfGeoTransform[3],
                    adfGeoTransform[4],
                    adfGeoTransform[5],
                )
            )

    # --------------------------------------------------------------------
    #      Report GCPs.
    # --------------------------------------------------------------------
    if bShowGCPs and hDataset.GetGCPCount() > 0:

        pszProjection = hDataset.GetGCPProjection()
        if pszProjection is not None:

            hSRS = osr.SpatialReference()
            if hSRS.ImportFromWkt(pszProjection) == gdal.CE_None:
                pszPrettyWkt = hSRS.ExportToPrettyWkt(False)
                print("GCP Projection = \n%s" % pszPrettyWkt)

            else:
                print("GCP Projection = %s" % pszProjection)

        gcps = hDataset.GetGCPs()
        i = 0
        for gcp in gcps:

            print(
                "GCP[%3d]: Id=%s, Info=%s\n"
                "          (%.15g,%.15g) -> (%.15g,%.15g,%.15g)"
                % (
                    i,
                    gcp.Id,
                    gcp.Info,
                    gcp.GCPPixel,
                    gcp.GCPLine,
                    gcp.GCPX,
                    gcp.GCPY,
                    gcp.GCPZ,
                )
            )
            i = i + 1

    # --------------------------------------------------------------------
    #      Report metadata.
    # --------------------------------------------------------------------
    if bShowMetadata:
        papszMetadata = hDataset.GetMetadata_List()
    else:
        papszMetadata = None
    if bShowMetadata and papszMetadata:
        print("Metadata:")
        for metadata in papszMetadata:
            print("  %s" % metadata)

    if bShowMetadata:
        for extra_domain in papszExtraMDDomains:
            papszMetadata = hDataset.GetMetadata_List(extra_domain)
            if papszMetadata:
                print("Metadata (%s):" % extra_domain)
                for metadata in papszMetadata:
                    print("  %s" % metadata)

    # --------------------------------------------------------------------
    #      Report "IMAGE_STRUCTURE" metadata.
    # --------------------------------------------------------------------
    if bShowMetadata:
        papszMetadata = hDataset.GetMetadata_List("IMAGE_STRUCTURE")
    else:
        papszMetadata = None
    if bShowMetadata and papszMetadata:
        print("Image Structure Metadata:")
        for metadata in papszMetadata:
            print("  %s" % metadata)

    # --------------------------------------------------------------------
    #      Report subdatasets.
    # --------------------------------------------------------------------
    papszMetadata = hDataset.GetMetadata_List("SUBDATASETS")
    if papszMetadata:
        print("Subdatasets:")
        for metadata in papszMetadata:
            print("  %s" % metadata)

    # --------------------------------------------------------------------
    #      Report geolocation.
    # --------------------------------------------------------------------
    if bShowMetadata:
        papszMetadata = hDataset.GetMetadata_List("GEOLOCATION")
    else:
        papszMetadata = None
    if bShowMetadata and papszMetadata:
        print("Geolocation:")
        for metadata in papszMetadata:
            print("  %s" % metadata)

    # --------------------------------------------------------------------
    #      Report RPCs
    # --------------------------------------------------------------------
    if bShowMetadata:
        papszMetadata = hDataset.GetMetadata_List("RPC")
    else:
        papszMetadata = None
    if bShowMetadata and papszMetadata:
        print("RPC Metadata:")
        for metadata in papszMetadata:
            print("  %s" % metadata)

    # --------------------------------------------------------------------
    #      Setup projected to long/lat transform if appropriate.
    # --------------------------------------------------------------------
    hProj = hDataset.GetSpatialRef()
    if hProj:
        hLatLong = hProj.CloneGeogCS()
        if hLatLong is not None:
            # To make sure the geographic coordinates are in long, lat order
            hLatLong.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
            gdal.PushErrorHandler("CPLQuietErrorHandler")
            hTransform = osr.CoordinateTransformation(hProj, hLatLong)
            gdal.PopErrorHandler()

    # --------------------------------------------------------------------
    #      Report corners.
    # --------------------------------------------------------------------
    print("Corner Coordinates:")
    GDALInfoReportCorner(hDataset, hTransform, "Upper Left", 0.0, 0.0)
    GDALInfoReportCorner(hDataset, hTransform, "Lower Left", 0.0, hDataset.RasterYSize)
    GDALInfoReportCorner(hDataset, hTransform, "Upper Right", hDataset.RasterXSize, 0.0)
    GDALInfoReportCorner(
        hDataset, hTransform, "Lower Right", hDataset.RasterXSize, hDataset.RasterYSize
    )
    GDALInfoReportCorner(
        hDataset,
        hTransform,
        "Center",
        hDataset.RasterXSize / 2.0,
        hDataset.RasterYSize / 2.0,
    )

    # ====================================================================
    #      Loop over bands.
    # ====================================================================
    for iBand in range(hDataset.RasterCount):

        hBand = hDataset.GetRasterBand(iBand + 1)

        # if( bSample )
        # {
        #    float afSample[10000];
        #    int   nCount;
        #
        #    nCount = GDALGetRandomRasterSample( hBand, 10000, afSample );
        #    print( "Got %d samples.\n", nCount );
        # }

        (nBlockXSize, nBlockYSize) = hBand.GetBlockSize()
        print(
            "Band %d Block=%dx%d Type=%s, ColorInterp=%s"
            % (
                iBand + 1,
                nBlockXSize,
                nBlockYSize,
                gdal.GetDataTypeName(hBand.DataType),
                gdal.GetColorInterpretationName(hBand.GetRasterColorInterpretation()),
            )
        )

        if hBand.GetDescription() is not None and hBand.GetDescription():
            print("  Description = %s" % hBand.GetDescription())

        dfMin = hBand.GetMinimum()
        dfMax = hBand.GetMaximum()
        if dfMin is not None or dfMax is not None or bComputeMinMax:

            line = "  "
            if dfMin is not None:
                line = line + ("Min=%.3f " % dfMin)
            if dfMax is not None:
                line = line + ("Max=%.3f " % dfMax)

            if bComputeMinMax:
                gdal.ErrorReset()
                adfCMinMax = hBand.ComputeRasterMinMax(False)
                if gdal.GetLastErrorType() == gdal.CE_None:
                    line = line + (
                        "  Computed Min/Max=%.3f,%.3f" % (adfCMinMax[0], adfCMinMax[1])
                    )

            print(line)

        stats = hBand.GetStatistics(bApproxStats, bStats)
        # Before GDAL 3.10, a negative value for stddev indicated an error
        if stats is not None and stats[3] >= 0.0:
            print(
                "  Minimum=%.3f, Maximum=%.3f, Mean=%.3f, StdDev=%.3f"
                % (stats[0], stats[1], stats[2], stats[3])
            )

        if bReportHistograms:

            hist = hBand.GetDefaultHistogram(
                force=True, callback=gdal.TermProgress_nocb
            )
            if hist is not None:
                dfMin = hist[0]
                dfMax = hist[1]
                nBucketCount = hist[2]
                panHistogram = hist[3]

                print("  %d buckets from %g to %g:" % (nBucketCount, dfMin, dfMax))
                line = "  "
                for bucket in panHistogram:
                    line = line + ("%d " % bucket)

                print(line)

        if bComputeChecksum:
            print("  Checksum=%d" % hBand.Checksum())

        dfNoData = hBand.GetNoDataValue()
        if dfNoData is not None:
            if math.isnan(dfNoData):
                print("  NoData Value=nan")
            else:
                print("  NoData Value=%.18g" % dfNoData)

        if hBand.GetOverviewCount() > 0:

            line = "  Overviews: "
            for iOverview in range(hBand.GetOverviewCount()):

                if iOverview != 0:
                    line = line + ", "

                hOverview = hBand.GetOverview(iOverview)
                if hOverview is not None:

                    line = line + ("%dx%d" % (hOverview.XSize, hOverview.YSize))

                    pszResampling = hOverview.GetMetadataItem("RESAMPLING", "")

                    if (
                        pszResampling is not None
                        and len(pszResampling) >= 12
                        and EQUAL(pszResampling[0:12], "AVERAGE_BIT2")
                    ):
                        line = line + "*"

                else:
                    line = line + "(null)"

            print(line)

            if bComputeChecksum:

                line = "  Overviews checksum: "
                for iOverview in range(hBand.GetOverviewCount()):

                    if iOverview != 0:
                        line = line + ", "

                    hOverview = hBand.GetOverview(iOverview)
                    if hOverview is not None:
                        line = line + ("%d" % hOverview.Checksum())
                    else:
                        line = line + "(null)"
                print(line)

        if hBand.HasArbitraryOverviews():
            print("  Overviews: arbitrary")

        nMaskFlags = hBand.GetMaskFlags()
        if (nMaskFlags & (gdal.GMF_NODATA | gdal.GMF_ALL_VALID)) == 0:

            hMaskBand = hBand.GetMaskBand()

            line = "  Mask Flags: "
            if (nMaskFlags & gdal.GMF_PER_DATASET) != 0:
                line = line + "PER_DATASET "
            if (nMaskFlags & gdal.GMF_ALPHA) != 0:
                line = line + "ALPHA "
            if (nMaskFlags & gdal.GMF_NODATA) != 0:
                line = line + "NODATA "
            if (nMaskFlags & gdal.GMF_ALL_VALID) != 0:
                line = line + "ALL_VALID "
            print(line)

            if hMaskBand is not None and hMaskBand.GetOverviewCount() > 0:

                line = "  Overviews of mask band: "
                for iOverview in range(hMaskBand.GetOverviewCount()):

                    if iOverview != 0:
                        line = line + ", "

                    hOverview = hMaskBand.GetOverview(iOverview)
                    if hOverview is not None:
                        line = line + ("%d" % hOverview.Checksum())
                    else:
                        line = line + "(null)"

        if hBand.GetUnitType():
            print("  Unit Type: %s" % hBand.GetUnitType())

        papszCategories = hBand.GetRasterCategoryNames()
        if papszCategories is not None:

            print("  Categories:")
            i = 0
            for category in papszCategories:
                print("    %3d: %s" % (i, category))
                i = i + 1

        scale = hBand.GetScale()
        if not scale:
            scale = 1.0
        offset = hBand.GetOffset()
        if not offset:
            offset = 0.0
        if scale != 1.0 or offset != 0.0:
            print("  Offset: %.15g,   Scale:%.15g" % (offset, scale))

        if bShowMetadata:
            papszMetadata = hBand.GetMetadata_List()
        else:
            papszMetadata = None
        if bShowMetadata and papszMetadata:
            print("  Metadata:")
            for metadata in papszMetadata:
                print("    %s" % metadata)

        if bShowMetadata:
            papszMetadata = hBand.GetMetadata_List("IMAGE_STRUCTURE")
        else:
            papszMetadata = None
        if bShowMetadata and papszMetadata:
            print("  Image Structure Metadata:")
            for metadata in papszMetadata:
                print("    %s" % metadata)

        hTable = hBand.GetRasterColorTable()
        if (
            hBand.GetRasterColorInterpretation() == gdal.GCI_PaletteIndex
            and hTable is not None
        ):

            print(
                "  Color Table (%s with %d entries)"
                % (
                    gdal.GetPaletteInterpretationName(
                        hTable.GetPaletteInterpretation()
                    ),
                    hTable.GetCount(),
                )
            )

            if bShowColorTable:

                for i in range(hTable.GetCount()):
                    sEntry = hTable.GetColorEntry(i)
                    print(
                        "  %3d: %d,%d,%d,%d"
                        % (i, sEntry[0], sEntry[1], sEntry[2], sEntry[3])
                    )

        if bShowRAT:
            pass
            # hRAT = hBand.GetDefaultRAT()

            # GDALRATDumpReadable( hRAT, None );

    return 0


# **********************************************************************
#                        GDALInfoReportCorner()
# **********************************************************************


def GDALInfoReportCorner(hDataset, hTransform, corner_name, x, y):

    line = "%-11s " % corner_name

    # --------------------------------------------------------------------
    #      Transform the point into georeferenced coordinates.
    # --------------------------------------------------------------------
    adfGeoTransform = hDataset.GetGeoTransform(can_return_null=True)
    if adfGeoTransform is not None:
        dfGeoX = adfGeoTransform[0] + adfGeoTransform[1] * x + adfGeoTransform[2] * y
        dfGeoY = adfGeoTransform[3] + adfGeoTransform[4] * x + adfGeoTransform[5] * y

    else:
        line = line + ("(%7.1f,%7.1f)" % (x, y))
        print(line)
        return False

    # --------------------------------------------------------------------
    #      Report the georeferenced coordinates.
    # --------------------------------------------------------------------
    if abs(dfGeoX) < 181 and abs(dfGeoY) < 91:
        line = line + ("(%12.7f,%12.7f) " % (dfGeoX, dfGeoY))

    else:
        line = line + ("(%12.3f,%12.3f) " % (dfGeoX, dfGeoY))

    # --------------------------------------------------------------------
    #      Transform to latlong and report.
    # --------------------------------------------------------------------
    if hTransform is not None:
        pnt = hTransform.TransformPoint(dfGeoX, dfGeoY, 0)
        if pnt is not None:
            line = line + ("(%s," % gdal.DecToDMS(pnt[0], "Long", 2))
            line = line + ("%s)" % gdal.DecToDMS(pnt[1], "Lat", 2))

    print(line)

    return True


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13

import sys

# Running main() must be protected that way due to use of multiprocessing on Windows:
# https://docs.python.org/3/library/multiprocessing.html#the-spawn-and-forkserver-start-methods
if __name__ == "__main__":
    from osgeo.gdal import deprecation_warn

    # import osgeo_utils.gdal2tiles as a convenience to use as a script
    from osgeo_utils.gdal2tiles import *  # noqa
    from osgeo_utils.gdal2tiles import main

    deprecation_warn("gdal2tiles")
    sys.exit(main(sys.argv, called_from_main=True))



#!/usr/local/bin/python3.13

import sys

from osgeo.gdal import deprecation_warn

# import osgeo_utils.ogrmerge as a convenience to use as a script
from osgeo_utils.ogrmerge import *  # noqa
from osgeo_utils.ogrmerge import main

deprecation_warn("ogrmerge")
sys.exit(main(sys.argv))



#!/usr/local/bin/python3.13
# -*- coding: utf-8 -*-
# ******************************************************************************
#
#  Project:  GDAL
#  Purpose:  Create a 2-level hierarchy of VRTs for very large collections
#  Author:   Even Rouault <even dot rouault at spatialys.com>
#
# ******************************************************************************
#  Copyright (c) 2024, Even Rouault <even dot rouault at spatialys.com>
#
# SPDX-License-Identifier: MIT
# ******************************************************************************

import glob
import math
import os
import sys

from osgeo import gdal, ogr
from osgeo_utils.auxiliary.gdal_argparse import GDALArgumentParser, GDALScript

gdal.UseExceptions()


class GDALBuildVRTOfVRT(GDALScript):
    def __init__(self):
        super().__init__()
        self.title = "Build a VRT of VRTs"
        self.description = (
            "Create a 2-level hierarchy of VRTs for very large collections"
        )

        self.optfile_arg = "--optfile"

    def get_parser(self, argv) -> GDALArgumentParser:
        parser = self.parser

        parser.add_argument("out_vrtfile", help="output VRT file")
        parser.add_argument("in_files", nargs="+", help="input files")

        parser.add_argument(
            "--max-files-per-vrt",
            dest="max_files_per_vrt",
            type=int,
            default=1000,
            metavar="number",
            help="maximum number of files per VRT",
        )

        parser.add_argument(
            "--intermediate-vrt-path",
            dest="intermediate_vrt_path",
            type=str,
            help="path where to put intermediate VRTs",
        )

        parser.add_argument(
            "-addalpha",
            action="store_true",
            help="whether to add an alpha channel",
        )

        parser.add_argument(
            "-tr",
            type=float,
            metavar="<res>",
            nargs=2,
            help="target resolution",
        )

        parser.add_argument(
            "-r",
            dest="resampling_alg",
            type=str,
            metavar="{nearest|bilinear|cubic|cubicspline|lanczos|average|mode}",
            help="resampling algorithm",
        )

        parser.add_argument(
            "--stop-on-error",
            dest="stop_on_error",
            action="store_true",
            help="whether an error when opening a source file should stop the whole process (by default processing continues skipping it)",
        )

        parser.add_argument(
            "--intermediate-vrt-add-overviews",
            dest="intermediate_vrt_add_overviews",
            action="store_true",
            help="whether overviews should be generated on intermediate VRTs (overview factors automatically determined)",
        )

        def list_of_ints(arg):
            return list(map(int, arg.split(",")))

        parser.add_argument(
            "--intermediate-vrt-overview-factors",
            dest="intermediate_vrt_overview_factors",
            type=list_of_ints,
            metavar="<factor1>[,<factor2>]...",
            help="Ask for overviews to be generated on intermediated VRTs and specify overview factor(s)",
        )

        def list_of_ints(arg):
            return list(map(int, arg.split(",")))

        parser.add_argument(
            "--overview-compression",
            dest="overview_compression",
            type=str,
            choices=("NONE", "LZW", "DEFLATE", "ZSTD", "JPEG", "LERC", "JXL"),
            default="LZW",
            help="overview compression algorithm",
        )

        return parser

    def doit(self, **kwargs):
        out_vrtfile = kwargs["out_vrtfile"]
        assert out_vrtfile.endswith(".vrt")
        max_files_per_vrt = kwargs["max_files_per_vrt"]

        global_srs = "invalid"
        global_minx = float("inf")
        global_miny = float("inf")
        global_maxx = float("-inf")
        global_maxy = float("-inf")
        catalog = []

        tmp_index_ds = gdal.GetDriverByName("GPKG").Create(
            ":memory:", 0, 0, gdal.GDT_Unknown
        )
        lyr = tmp_index_ds.CreateLayer("index")
        lyr.CreateField(ogr.FieldDefn("filename", ogr.OFTString))

        print("Initial pass...")
        source_files = []
        for in_file in kwargs["in_files"]:
            if "*" in in_file:
                source_files += glob.glob(in_file)
            else:
                source_files.append(in_file)

        for in_file in source_files:

            def deal_with_error(msg):
                if kwargs["stop_on_error"]:
                    raise Exception(f"Error on {in_file}: {msg}")
                else:
                    print(f"Skipping {in_file}. {msg}")

            try:
                ds = gdal.Open(in_file)
            except Exception as e:
                deal_with_error(f"{e}")
                continue
            gt = ds.GetGeoTransform(can_return_null=True)
            if gt is None:
                deal_with_error("No geotransform.")
                continue
            if gt[5] > 0:
                deal_with_error("South-up rasters are not supported.")
                continue
            if gt[2] != 0 or gt[4] != 0:
                deal_with_error("Rotated rasters are not supported.")
                continue

            srs = ds.GetSpatialRef()
            if global_srs == "invalid":
                global_srs = srs
            else:
                if global_srs is not None and srs is None:
                    print(
                        f"Skipping {in_file}. It has no CRS, whereas other files have one."
                    )
                    continue
                if global_srs is None and srs is not None:
                    print(
                        f"Skipping {in_file}. It has a CRS, whereas other files have not one."
                    )
                    continue
                if (
                    global_srs is not None
                    and srs is not None
                    and not global_srs.IsSame(srs)
                ):
                    print(
                        f"Skipping {in_file}. It has a different CRS compared to the other files."
                    )
                    continue

            minx = gt[0]
            maxx = gt[0] + gt[1] * ds.RasterXSize
            maxy = gt[3]
            miny = gt[3] + gt[5] * ds.RasterYSize
            catalog.append((in_file, minx, miny, maxx, maxy))

            global_minx = min(global_minx, minx)
            global_miny = min(global_miny, miny)
            global_maxx = max(global_maxx, maxx)
            global_maxy = max(global_maxy, maxy)

            f = ogr.Feature(lyr.GetLayerDefn())
            f["filename"] = in_file
            g = ogr.CreateGeometryFromWkt(
                f"POLYGON(({minx} {miny},{minx} {maxy},{maxx} {maxy},{maxx} {miny},{minx} {miny}))"
            )
            f.SetGeometry(g)
            lyr.CreateFeature(f)

        number_of_files = len(catalog)
        if number_of_files == 0:
            raise Exception("No source files have been found!")

        number_of_vrts = int(math.ceil(number_of_files / max_files_per_vrt))
        sqrt_number_of_vrts = int(math.ceil(number_of_vrts**0.5))
        ratio_x_over_y = (global_maxx - global_minx) / (global_maxy - global_miny)
        num_vrt_along_x = int(math.ceil(sqrt_number_of_vrts / (ratio_x_over_y**0.5)))
        num_vrt_along_y = int(math.ceil(sqrt_number_of_vrts * (ratio_x_over_y**0.5)))

        stepx = (global_maxx - global_minx) / num_vrt_along_x
        stepy = (global_maxy - global_miny) / num_vrt_along_y

        vrt_options = ""

        if kwargs["addalpha"]:
            vrt_options += " -addalpha"

        tr = kwargs["tr"]
        if tr:
            vrt_options += f" -tr {tr[0]} {tr[1]}"

        resampling_alg = kwargs["resampling_alg"]
        if resampling_alg:
            vrt_options += f" -r {resampling_alg}"

        intermediate_vrt_path = kwargs["intermediate_vrt_path"]

        vrt_files = []
        for j in range(num_vrt_along_y):
            for i in range(num_vrt_along_x):
                if intermediate_vrt_path:
                    vrt_filename = os.path.join(
                        intermediate_vrt_path,
                        os.path.basename(out_vrtfile)[0:-4] + f"_{i}_{j}.vrt",
                    )
                else:
                    vrt_filename = out_vrtfile[0:-4] + f"_{i}_{j}.vrt"
                minx = global_minx + i * stepx
                maxx = global_minx + (i + 1) * stepx
                miny = global_miny + j * stepy
                maxy = global_miny + (j + 1) * stepy
                lyr.SetSpatialFilterRect(minx, miny, maxx, maxy)
                subvrt_files = [f["filename"] for f in lyr]
                if subvrt_files:
                    pct = (
                        100.0
                        * (j * num_vrt_along_x + i)
                        / (num_vrt_along_x * num_vrt_along_y)
                    )
                    print(f"Building {vrt_filename} (%.02f %%)..." % pct)
                    vrt_files.append(vrt_filename)
                    gdal.BuildVRT(vrt_filename, subvrt_files, options=vrt_options)

                    if kwargs["intermediate_vrt_overview_factors"]:
                        print(f"Building {vrt_filename}.ovr (%.02f %%)..." % pct)
                        ds = gdal.Open(vrt_filename)
                        with gdal.config_option(
                            "COMPRESS_OVERVIEW", kwargs["overview_compression"]
                        ):
                            ds.BuildOverviews(
                                kwargs["resampling_alg"],
                                kwargs["intermediate_vrt_overview_factors"],
                            )
                    elif kwargs["intermediate_vrt_add_overviews"]:
                        ds = gdal.Open(vrt_filename)
                        factors = []
                        max_dim = max(ds.RasterXSize, ds.RasterYSize)
                        factor = 2
                        while max_dim > 256:
                            factors.append(factor)
                            max_dim = max_dim // 2
                            factor *= 2
                        if factors:
                            print(f"Building {vrt_filename}.ovr (%.02f %%)..." % pct)
                            with gdal.config_option(
                                "COMPRESS_OVERVIEW", kwargs["overview_compression"]
                            ):
                                ds.BuildOverviews(kwargs["resampling_alg"], factors)

        print(f"Building {out_vrtfile} (100 %)...")
        gdal.BuildVRT(out_vrtfile, vrt_files, options=vrt_options)


def main(argv=sys.argv):
    return GDALBuildVRTOfVRT().main(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv))
